# -*- coding: utf-8 -*-

import os, sys
# ensure citruleaf.py is in path
sys.path.insert(1, os.path.realpath( os.pardir ) )

import time
import unittest
import citrusleaf as cl

class TestCreateUser(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        
        cls.cluster_with_user_admin = cl.CitrusleafCluster()
        cls.cluster_with_user_admin.setUser('testuauser', cl.hashpassword('aerospike'))
        cl.cluster_setHost(cls.cluster_with_user_admin, 'localhost', '3000')

        cls.cluster_with_read_user = cl.CitrusleafCluster()
        cls.cluster_with_read_user.setUser('testreaduser', cl.hashpassword('aerospike'))
        cl.cluster_setHost(cls.cluster_with_read_user, 'localhost', '3000')
    
    @classmethod
    def tearDownClass(cls):

        del(cls.cluster_with_user_admin)
        del(cls.cluster_with_read_user)
   
    
    def test_createUser_and_dropUser(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'User1', cl.hashpassword('aerospike'), ["read", "read-write"])

        self.assertEqual(0, status)

        time.sleep(1)

        status = cl.dropUser(TestCreateUser.cluster_with_user_admin, 'User1')

        self.assertEqual(0, status)

    def test_createUser_for_already_present_user(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'admin', cl.hashpassword('admin'))

        self.assertEqual(61, status)

    def test_createUser_for_empty_username(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, '', cl.hashpassword('admin'))

        self.assertEqual(60, status)

    def test_createUser_for_empty_password(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'User2', cl.hashpassword(''))

        self.assertEqual(0, status)

        time.sleep(1)

        status = cl.dropUser(TestCreateUser.cluster_with_user_admin, 'User2')

        self.assertEqual(0, status)

    def test_createUser_with_no_roles(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'User3', cl.hashpassword('password'))

        self.assertEqual(0, status)

        time.sleep(1)

        status = cl.dropUser(TestCreateUser.cluster_with_user_admin, 'User3')

        self.assertEqual(0, status)
    
    def test_createUser_with_empty_roles_list(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'User4', cl.hashpassword('password'), roles=[])

        self.assertEqual(-1, status)
    
    def test_createUser_with_other_than_useradmin_user(self):
 
        status = cl.createUser(TestCreateUser.cluster_with_read_user, 'User5', cl.hashpassword('password'), roles=["read"])

        self.assertEqual(81, status)

    def test_createUser_for_more_than_one_roles(self):

        status = cl.createUser(TestCreateUser.cluster_with_user_admin, 'User5', cl.hashpassword('p@ssword'), roles=["read", "read-write", "user-admin"])
        
        self.assertEqual(0, status)

        time.sleep(1)

        user = cl.queryUser(TestCreateUser.cluster_with_user_admin, 'User5')

        self.assertEqual(0, user[0])
        self.assertEqual('User5', user[1].name)
        self.assertEqual(["read", "read-write", "user-admin"].sort(), user[1].roles.sort())

        cl.dropUser(TestCreateUser.cluster_with_user_admin, 'User5')

    def test_createUser_with_other_than_useradmin_but_with_more_than_one_roles(self):

        status = cl.grantRoles(TestCreateUser.cluster_with_user_admin, 'testreaduser', roles=['read-write', 'sys-admin'])

        self.assertEqual(0, status)

        status = cl.createUser(TestCreateUser.cluster_with_read_user, 'User5', cl.hashpassword('p@ssword'), roles=['read', 'read-write'])

        self.assertEqual(81, status)

        cl.replaceRoles(TestCreateUser.cluster_with_user_admin, 'testreaduser', roles=['read'])


if __name__ == "__main__":
    unittest.main()
